@@ -219,15 +219,38 @@ class CompoundMeasure(Measure):
     their computed values. It can be used to annotate a tree with multiple
     measures at the same time.
     """
-    def __init__(self, *measures):
+    def __init__(self, *measures, **kwargs):
+        """
+        Creates a measure that combines the specified measures.
+        
+        One keyword argument, tuple_class, can be present. It specifies a
+        subclass of tuple to use to combine the measures' computed values. This
+        is primarily intended to allow named tuples (classes returned from
+        collections.namedtuple) to be used instead of plain vanilla tuples.
+        """
         self.measures = measures
-        self.identity = tuple(m.identity for m in self.measures)
+        if "tuple_class" in kwargs:
+            tuple_class = kwargs["tuple_class"]
+            # Unlike normal tuples, named tuples don't allow passing a single
+            # sequence argument into their constructor; they provide _make for
+            # this purpose. We'll want to store the tuple class's _make, then,
+            # instead of the class itself.
+            if hasattr(tuple_class, "_make"):
+                self._make_tuple = tuple_class._make
+            # If it doesn't have a _make, we assume that it's a custom subclass
+            # of tuple, and that it should be smart enough to accept a single
+            # sequence parameter.
+            else:
+                self._make_tuple = tuple_class
+        else:
+            self._make_tuple = tuple
+        self.identity = self._make_tuple(m.identity for m in self.measures)
     
     def convert(self, value):
-        return tuple(m.convert(value) for m in self.measures)
+        return self._make_tuple(m.convert(value) for m in self.measures)
     
     def operator(self, a_values, b_values):
-        return tuple(m.operator(a, b) for (m, a, b) in zip(self.measures, a_values, b_values))
+        return self._make_tuple(m.operator(a, b) for (m, a, b) in zip(self.measures, a_values, b_values))
 
 
 class _NodeMeasure(Measure):
@@ -244,6 +267,7 @@ MEASURE_ITEM_COUNT = MeasureItemCount()
 
 
 class Node(Sequence):
+    __slots__ = ["_values", "measure", "annotation"]
     def __init__(self, measure, *values):
         if len(values) not in (2, 3):
             raise Exception("Nodes must have 2 or 3 children")
@@ -268,6 +292,7 @@ class Node(Sequence):
 
 
 class Digit(Sequence):
+    __slots__ = ["_values", "measure", "annotation"]
     def __init__(self, measure, *values):
         if len(values) not in (1, 2, 3, 4):
             raise Exception("Digits must have 1, 2, 3, or 4 children; the "
@@ -309,6 +334,15 @@ class Digit(Sequence):
         return "<Digit: %s>" % ", ".join([repr(v) for v in self])
 
 
+# Note: We have a __slots__ on Node and Digit to reduce their memory footprint,
+# but adding __slots__ to Tree and its subclasses would be premature
+# optimization: a 2-3 finger tree containing n values will have at most
+# log2(n) instances of any of Tree's subclasses, so we'll get little benefit
+# in the long run.
+# 
+# I might, however, consider adding __slots__ for the sake of applications
+# where lots of little trees are used.
+
 class Tree(object):
     """
     A class representing a 2-3 finger tree.
